<?php

declare(strict_types=1);

namespace Plugin\cin_events_5;

require_once __DIR__.'/vendor/autoload.php';

use Cin\Events5\Licence;
use Cin\Events5\LicenceWidget;
use Cin\Events5\Plugin;
use JTL\Backend\NotificationEntry;
use JTL\Events\Dispatcher;
use JTL\Plugin\Bootstrapper;
use JTL\Shop;
use PDO;

final class Bootstrap extends Bootstrapper
{
  private $P;

  public function boot(Dispatcher $dispatcher): void
  {
    parent::boot($dispatcher);
    // listen for admin calls
    if (0 === Shop::getPageType() && str_starts_with($_SERVER['REQUEST_URI'], URL_SHOP.'/admin/')) {
      $L = new Licence();
      if ( ! $L->canRun()) {
        $this->licenceNotification();
      }
    }

    Shop::Container()->getGetText()->setLanguage()->loadTranslations($this->P()->getJTlPlugin()->getPaths()->getBasePath(), 'custom');
  }

  public function enabled(): void
  {
    LicenceWidget::initializeWidget();
  }

  public function installed(): void
  {
    LicenceWidget::initializeWidget();
  }

  public function uninstalled(bool $deleteData = true): void
  {
    LicenceWidget::checkRemoveWidget();
    $this->unsetSession();
  }

  public function updated($oldVersion, $newVersion): void
  {
    LicenceWidget::initializeWidget();
    $this->unsetSession();
  }

  private function getSettingsTabID(string $identifier): string
  {
    $pdo  = $this->P()->getPDO();
    $prep = $pdo->prepare('SELECT kPluginAdminMenu FROM tpluginadminmenu WHERE kPlugin = :kPlugin AND cName = :cName');

    $prep->execute([
      'kPlugin' => $this->getPlugin()->getID(),
      'cName'   => $identifier
    ]);

    return (string) $prep->fetch(PDO::FETCH_COLUMN) ?: 0;
  }

  private function licenceNotification(): void
  {
    $this->addNotify(
      NotificationEntry::TYPE_DANGER,
      $this->getPlugin()->getMeta()->getName(),
      '<a href="https://www.com-ins-netz.de/Support-Hilfecenter">Es gibt Probleme mit der Lizenz. Wenden Sie sich an com-ins-netz.de</a>'
    );
  }

  private function P(): Plugin
  {
    return $this->P ?? ($this->P = new Plugin());
  }

  private function unsetSession(): void
  {
    unset(
      $_SESSION["{$this->getPlugin()->getPluginID()}_versionResponse"],
      $_SESSION["{$this->getPlugin()->getPluginID()}_productHomepage"]
    );
  }
}
