<?php

declare(strict_types=1);

namespace Plugin\cin_events_5\Portlets\EventListe;

require \dirname(__DIR__, 2).'/src/warmup.php';

use Cin\Events5\PortletTrait;
use JTL\Catalog\Product\Artikel;
use JTL\Helpers\Product;
use JTL\OPC\InputType;
use JTL\OPC\Portlet;
use JTL\OPC\PortletInstance;
use JTL\Shop;
use PDO;

final class EventListe extends Portlet
{
  use PortletTrait;

  public function getButtonHtml(): string
  {
    return '<i class="fas fa-icons" '
      .(
        $this->isLicensed($this->class) && 'events' === trim(Shop::Smarty()->tpl_vars['page']->value->getUrl(), '/')
        ? 'style="color:#e87d0c'
        : 'style="color:#6c757d'
      )
    .'"></i><span>'.$this->getTitle().'</span>';
  }

  public function getPropertyDesc(): array
  {
    return []; // TODO DISABLED

    return [
      'hint-for-events' => [
        'label' => __('note'),
        'type'  => InputType::HINT,
        'class' => 'warning',
        'text'  => __('hint-for-events'),
      ],
      'listStyle' => [
        'type'    => InputType::SELECT,
        'label'   => __('presentation'),
        'width'   => 66,
        'options' => [
          'gallery'      => __('presentationGallery'),
          'list'         => __('presentationList'),
          'simpleSlider' => __('presentationSimpleSlider'),
          'slider'       => __('presentationSlider'),
          'box-slider'   => __('presentationBoxSlider'),
        ],
        'default' => 'slider',
      ],
      'maxProducts' => [
        'type'     => InputType::NUMBER,
        'label'    => __('maxProducts'),
        'width'    => 33,
        'default'  => 16,
        'min'      => 0,
        'required' => true,
      ],
      'choice_mode' => [
        'type'    => InputType::SELECT,
        'label'   => __('choice_mode'),
        'width'   => 49,
        'default' => 'all',
        'options' => [
          'all'       => __('all_events'),
          'specified' => __('specified_event')
        ],
        'childrenFor' => [
          'specified' => [
            'specified_category' => [
              'label' => __('displaying_specified_categorie'),
              'type'  => InputType::TEXT,
              'width' => 49,
            ]
          ]
        ]
      ],
    ];
  }

  public function getPropertyTabs(): array
  {
    return [
      __('Styles') => 'styles',
    ];
  }

  public function getConfigPanelHtml(PortletInstance $instance): string
  {
    $cTpl = parent::getConfigPanelHtml($instance);

    return $cTpl.$this->extendConfigDialog($instance)."
    <script>
      $('#childrenFor-selected-article_mode .col-4').toggleClass('col-4 col-12');
      $('#childrenFor-selected-article_mode .form-group').addClass('row');
      $('#childrenFor-selected-article_mode .input-group').addClass('col-4 row_');
      $('#childrenFor-selected-article_mode .form-group label:first-child').addClass('col-12 row_');
    </script>";
  }

  public function getEventListTemplate(PortletInstance $instance): string
  {
    if ( ! $this->isLicensed($this->class)) {
      return '';
    }

    $template = '';
    $style    = null;

    switch ($instance->getProperty('listStyle')) {
      case 'list':
        $style = 'list';
        $item  = ''; //(new JTLSmarty())->fetch(__DIR__ . '/loadListItem.tpl');
        //jdump($item);
        // no break
      case 'gallery':
        $template = '<div class="col product-wrapper mb-4 col-md- col-xl- col-12" itemprop="itemListElement" itemscope=true itemtype="https://schema.org/Product">';
        $template .= '  <div class="productbox productbox-row productbox-show-variations  productbox-hover" data-wrapper="true">';
        $template .= '    <div class="productbox-inner">';
        $template .= '      <div class="row ">';
        $template .= '        <div class="col  col-md-4 col-lg-6 col-xl-3 col-12">';
        $template .= '          <div class="productbox-image">';
        $template .= '            <div class="ribbon ribbon-8 productbox-ribbon">Auf Lager</div>'; // TODO
        $template .= '            <div class="productbox-images" data-size="s" data-direction-h="u" data-direction-w="l">';
        $template .= '              <a href="https://shop.braesie.dev.fu/Ticket-Erwachsener">';
        $template .= '                <div class="list-gallery">';
        $template .= '                  <div class="productbox-image square square-image first-wrapper">';
        $template .= '                    <div class="inner">';
        $template .= '                      <img class="  " height="auto" width="auto" alt="Ticket Erwachsener" src="https://shop.braesie.dev.fu/gfx/keinBild.gif">';
        $template .= '                    </div>';
        $template .= '                  </div>';
        $template .= '                </div>';
        $template .= '              </a>';
        $template .= '              <meta itemprop="image" content="https://shop.braesie.dev.fu/gfx/keinBild.gif">';
        $template .= '            </div>';
        $template .= '          </div>';
        $template .= '        </div>';
        $template .= '        <div class="col  col-md-">';
        $template .= '          <div class="productbox-title" itemprop="name">';
        $template .= '            <a href="https://shop.braesie.dev.fu/Ticket-Erwachsener">Ticket Erwachsener</a>';
        $template .= '          </div>';
        $template .= '          <meta itemprop="url" content="https://shop.braesie.dev.fu/Ticket-Erwachsener">';
        $template .= '          <div class="row ">';
        $template .= '            <div class="col productbox-options col-xl-4 col-12" itemprop="offers" itemscope="" itemtype="https://schema.org/Offer">';
        $template .= '              <link itemprop="businessFunction" href="http://purl.org/goodrelations/v1#Sell">';
        $template .= '              <div class="item-delivery-status delivery-status">';
        $template .= '                <div class="signal_image status-1">Knapper Lagerbestand</div>';
        $template .= '                <div class="estimated_delivery">Lieferzeit: 2 - 3 Werktage</div>';
        $template .= '              </div>';
        $template .= '            </div>';
        $template .= '          </div>';
        $template .= '        </div>';
        $template .= '      </div>';
        $template .= '    </div>';
        $template .= '  </div>';
        $template .= '</div>';

        return htmlspecialchars($template);

        break;
    }

    return '<div class="bullshit"></div>';
  }

  public function getFilteredProducts(PortletInstance $instance): array
  {
    if ( ! $this->isLicensed($this->class)) {
      return [];
    }

    $products = [];

    foreach ($this->loadArticleList($instance) as $product) {
      $products[] = (new Artikel())->fuelleArtikel((int) $product['kArtikel']);
    }

    return \array_slice(Product::separateByAvailability($products), 0, $instance->getProperty('maxProducts') ?: null);
  }

  private function appendCategoryFilter(string $mode, PortletInstance $instance)
  {
    $pdo = Shop::Container()->getDB()->getPDO();

    switch ($mode) {
      case 'selected':
        return "\nAND tk.cName IN (".implode(',', array_fill(0, \count($instance->getProperty('selecting_categories')), '?')).')';

        break;

      case 'extended':
        return "\nAND tk.kKategorie IN (SELECT * FROM (
          SELECT kKategorie
          FROM
            (SELECT * FROM tkategorie
            ORDER BY kOberKategorie, kKategorie) products_sorted,
            (SELECT @pv := (SELECT kKategorie FROM tkategorie WHERE cName LIKE ?)) initialisation
          WHERE   find_in_set(kOberKategorie, @pv)
          AND     length(@pv := concat(@pv, ',', kKategorie))
          ) AS subquery )";

        break;

      case 'all':
      default:
        return '';
    }
  }

  private function loadArticleList(PortletInstance $instance): array
  {
    $mode = $instance->getProperty('article_mode');
    $db   = Shop::Container()->getDB();

    $stmt = 'SELECT
      taa.kArtikel kArtikel,
      ta.cName Artikel,
      tk.cName Kategorie,
      tk.kKategorie kKategorie,
      tk.kOberKategorie parent,
      CAST(taa.cWert AS integer) sort
      FROM tartikelattribut taa
      LEFT JOIN tartikel ta ON ta.kArtikel = taa.kArtikel
      LEFT JOIN tkategorieartikel tka ON tka.kArtikel = taa.kArtikel
      LEFT JOIN tkategorie tk ON tk.kKategorie = tka.kKategorie
      WHERE ';

    $stmt .= ('extended' === $mode && $instance->getProperty('withoutAttribute')
      ? '1 = 1'
      : "taa.cName LIKE 'cin_homepage_article'");

    $prep = $db->getPDO()->prepare($stmt.$this->appendCategoryFilter($mode, $instance));

    if ('selected' === $mode) {
      foreach ($instance->getProperty('selecting_categories') as $k => $v) {
        $prep->bindValue(($k + 1), $v);
      }
    }

    $prep->execute('extended' === $mode ? [$instance->getProperty('extending_category')] : null);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }
}
