<?php

declare(strict_types=1);

use Cin\Events5\ColorMan;
use Cin\Events5\service\EventConfigService;
use Cin\Events5\service\EventMatrixService;
use Cin\Events5\service\EventService;

require_once \dirname(__DIR__, 3).'/admin/includes/admininclude.php';
require \dirname(__DIR__, 1).'/src/warmup.php';

// @var JTL\Backend\AdminAccount $oAccount

$oAccount->permission('PLUGIN_ADMIN_VIEW', true, true);

$response = $P->Response();

if ( ! $L->canRun()) {
  $response->status  = 'error';
  $response->message = 'licence failure';
} else {
  $jtlPlugin = $P->getJTlPlugin();
  $params    = array_merge($_GET, $_POST);
  if ('application/json' === (getallheaders()['Content-Type'] ?? '')) {
    $params = array_merge($params, json_decode(file_get_contents('php://input'), true) ?? []);
  }

  header_remove('Content-Type');
  switch ($_SERVER['REQUEST_METHOD'].'_'.($params['action'] ?? 'missingaction')) {
    case 'POST_delete': // TODO Rename to clarify name

      if (file_exists(PFAD_ROOT.$params['path'])) {
        if (unlink(PFAD_ROOT.$params['path'])) {
          $response->data->src = EventService::EVENT_NO_IMAGE;
          EventService::deleteEventImage($params['event']);
        }
      }

      break;

    case 'DELETE_config':

      $response = EventConfigService::delete($params['id']);

      break;
    case 'DELETE_date':

      $response = EventConfigService::deleteDate($params['date'], $params['eventID']);

      break;
    case 'DELETE_matrix':

      $filename = $params['default'].'.svg';

      if (EventMatrixService::hasLinkedReservations($filename)) {
        $response->code    = 400;
        $response->message = __('Matrix is still bound to purchases');
      } elseif (EventMatrixService::isLinkedToConfig($filename)) {
        $response->code    = 400;
        $response->message = __('Matrix is linked to configurations');
      } else {
        $stmt = $P->getPDO()->prepare('DELETE FROM cin_events_5_matrix WHERE filename = :filename');
        if ($stmt->execute([':filename' => $filename])) {
          try {
            unlink(PFAD_ROOT.'mediafiles/cin_events_5/matrix/'.$filename);
          } catch (\Throwable $th) {
          }
          $response->message = __('Matrix deleted');
        }
      }

      break;
    case 'POST_eventConfig':

      $response->data->config     = EventConfigService::getEventConfig($params['eventID'], $params['date'], $params['begin'], $params['end']);
      $response->data->matrixlist = EventMatrixService::getMatrixList();
      $response->data->variations = EventService::getVariation($params['eventID']);

      array_walk($response->data->config, static function (&$config): void {
        $config['linkedVariations'] = EventMatrixService::getMatrixVar($config['id']);
        if (isset($config['filename'])) {
          $config['svg'] = file_get_contents(PFAD_ROOT.'mediafiles/cin_events_5/matrix/'.$config['filename']);
        }
      });

      break;

    case 'GET_matrixlist':
      $response->data->matrixlist = EventMatrixService::getMatrixList();
      array_walk($response->data->matrixlist, static function (&$matrix): void {
        $matrix['svg']       = file_get_contents(PFAD_ROOT.'mediafiles/cin_events_5/matrix/'.$matrix['filename']);
        $matrix['deletable'] = ! EventMatrixService::hasLinkedReservations($matrix['filename']);
      });

      break;

    case 'POST_linkSvg':

      EventMatrixService::linkSvg($params['svgID'], $params['data']);

      break;

    case 'POST_load':

      $struct = EventService::loadEventStructure(PDO::FETCH_GROUP | PDO::FETCH_ASSOC);

      $respArray = array_flip($params['dateList']);
      array_walk($respArray, static function (&$val): void {
        $val = [];
      });

      if (\array_key_exists($params['eventID'], $struct)) {
        foreach ($struct[$params['eventID']] as $evtObj) {
          if (\in_array($evtObj['date'], $params['dateList'], true)) {
            $entry                        = $evtObj['begin'].' - '.$evtObj['end'];
            $entry                        = preg_replace('/^ - $/', __('all-day'), $entry);
            $respArray[$evtObj['date']][] = $entry;
          }
        }
      }

      $response->data = array_values($respArray);

      break;

    case 'POST_loadDatesColumn':

      $list           = EventConfigService::getEventDays($params['eventID']);
      $response->data = $list;

      break;

    case 'POST_loadTimes':

      $list = EventConfigService::getEventTimes($params['eventID'], $params['date']);
      array_walk($list, static function (&$array): void {
        $array['value'] = '00:00:00' === $array['end']
        ? sprintf(__('from-'), mb_substr($array['begin'], 0, 5))
        : mb_substr($array['begin'], 0, 5).(
          null === $array['end']
          ? ''
          : ' - '.mb_substr($array['end'], 0, 5)
        );
      });
      $response->data = $list;

      break;

    case 'POST_submitModals':

      switch ($params['type']) {
        case 'newdate':
          $response->status = EventConfigService::addDate($params['kArtikel'], $params['data'], $response->message) ? 'success' : 'error';

          break;
        case 'editdate':
          $response = EventConfigService::updateDate($params['data']['forDate'], $params['data']['addDate'], $params['kArtikel']);

          break;
        case 'newtime':
          $response->status = EventConfigService::addTime($params['kArtikel'], $params['data'], $response->message) ? 'success' : 'error';

          break;
        case 'edittime':
          $response  = EventConfigService::updateConfig($params['data']['config'], null, null, 'begin', $params['data']['addTimeBegin']);
          $response2 = EventConfigService::updateConfig($params['data']['config'], null, null, 'end', $params['data']['addTimeEnd']);

          if ('error' === $response2->status) {
            $response->status = 'error';
          }

          if (mb_strlen($response2->message) > 0 && false === mb_strpos($response->message, $response2->message)) {
            $response->message .= (mb_strlen($response->message) ? "\n" : '').$response2->message;
          }

          if (\count((array) $response2->data) > 0) {
            if (0 === \count((array) $response->data)) {
              $response->data = $response2->data;
            } elseif (isset($response->data->slice)) {
              $response->data->slice[] = $response->data;
              $response->data->slice[] = $response2->data;
            }
          }

          break;
        case 'clonedate':
          // $response->status = EventConfigService::cloneDate($params['data']['addDate'], $params['kArtikel'], $params['data']['forDate']) ? 'success' : 'error';
          $response->status = EventConfigService::cloneDate($params['data']['addDate'], $params['kArtikel'], $params['data']['forDate']) ? 'success' : 'error';

          break;
        case 'clonetime':
          $response->status = EventConfigService::clone($params['data']['config'], $params['data']['addTimeBegin'], $params['data']['addTimeEnd']) ? 'success' : 'error';

          break;
      }

      break;

    case 'GET_template':

      $response->data = EventService::getTemplates($params['type'], $smarty);

      break;

    case 'POST_testbutton':

      if ( ! (file_exists(PFAD_ROOT.'cindev.json') && $j = json_decode(file_get_contents(PFAD_ROOT.'cindev.json')) && isset($j->testmode) && $j->testmode)) {
        http_response_code(403);
      }
      // else {
      // use here test code
      // $response->data = EventService::ensureEvent(1090);
      // }

      break;

    case 'PUT_updateConfig':

      $response = EventConfigService::updateConfig((int) $params['id'], $params['eventID'], $params['date'], $params['column'], $params['value']);

      if ('matrix_id' === $params['column']) {
        $mD                               = EventMatrixService::getEventMatrixData($params['id']);
        $response->data->svg              = isset($mD[0]['filename']) ? file_get_contents(\dirname(__DIR__, 3).'/mediafiles/cin_events_5/matrix/'.$mD[0]['filename']) : '';
        $response->data->linkedVariations = EventMatrixService::getMatrixVar($params['id']);
      }

      break;

    case 'PUT_updateMatrixVar':

      $response->status = EventMatrixService::updateMatrixVar($params['configID'], $params['column'], $params['value']) ? 'success' : 'error';

      break;
    case 'PUT_matrixname':

      $response->status = EventMatrixService::updateMatrixName($params['update'], $params['default']);

      break;

    case 'POST_upload':
      require_once \dirname(__DIR__, 1).'/src/ColorMan.php';

      $stmt = $P->getPDO()->prepare("SELECT
        cWert AS w,
        (
          SELECT cWert
          FROM teinstellungen
          WHERE cName = 'bilder_artikel_klein_hoehe'
        ) as h
        FROM teinstellungen
        WHERE cName = 'bilder_artikel_klein_breite'");
      $stmt->execute();
      $dims = $stmt->fetch(PDO::FETCH_ASSOC);

      $i = imagecreatefromstring(file_get_contents($_FILES['dropdoc']['tmp_name']));
      $s = max($dims['w'], $dims['h']);

      $t = ColorMan::scaleMax($i, $s);
      $t = ColorMan::paddToSquare($t);

      if (false === is_dir(PFAD_ROOT.EventService::EVENT_USER_IMAGE_PATH)) {
        mkdir(PFAD_ROOT.EventService::EVENT_USER_IMAGE_PATH);
      }
      $filename = isset($params['linkname']) ? $params['linkname'].'.png' : pathinfo($_FILES['dropdoc']['name'], \PATHINFO_FILENAME);
      $filepath = EventService::EVENT_USER_IMAGE_PATH.$filename;
      imagepng($t, PFAD_ROOT.$filepath);
      $response->data->src = $filepath.'?'.(int) (microtime(true) * 10000);

      EventService::storeEventImage((int) $params['linkname'], $filepath);

      break;

    case 'POST_upload-matrix':

      $c = file_get_contents($_FILES['dropdoc']['tmp_name']);

      if (false === is_dir(PFAD_ROOT.EventService::EVENT_USER_IMAGE_PATH)) {
        mkdir(PFAD_ROOT.EventService::EVENT_USER_IMAGE_PATH);
      }
      if (false === is_dir(PFAD_ROOT.EventService::EVENT_USER_IMAGE_PATH.'matrix/')) {
        mkdir(PFAD_ROOT.EventService::EVENT_USER_IMAGE_PATH.'matrix/');
      }

      $identifier = EventService::generateRandomString();

      // $filename = pathinfo($_FILES['dropdoc']['name'], PATHINFO_FILENAME).'.svg';
      $filename = $identifier.'.svg';
      $filepath = EventService::EVENT_USER_IMAGE_PATH.'matrix/'.$filename;

      $success = move_uploaded_file($_FILES['dropdoc']['tmp_name'], PFAD_ROOT.$filepath);

      $response->status = ($success && EventMatrixService::storeMatrix($identifier, $filename) ? 'success' : 'error');

      $response->message = 'success' === $response->status ? __('matrix_stored') : __('error_on_upload');

      if ('success' === $response->status) {
        $response->data         = new stdClass();
        $response->data->svg    = $c;
        $response->data->matrix = EventMatrixService::getMatrixByIdentifier($identifier);
      }

      if ('success' === $response->status && preg_match('/<\s*script/i', $c) > 0) {
        $response->status  = 'warn';
        $response->message = 'SVG enthält ein Script welches gefährliche Daten nachladen könnte!';
      }

      break;

    default:
      $response->status  = 'error';
      $response->message = "action '{$params['action']}' is not defined";
  }
}

$P->exitResponse($response);
