<?php

declare(strict_types=1);

use Cin\Events5\Licence;
use Cin\Events5\Plugin;
use Cin\Events5\service\EventCartService;
use Cin\Events5\service\EventConfigService;
use Cin\Events5\service\EventMatrixService;
use Cin\Events5\service\EventService;

require \dirname(__DIR__, 1).'/src/warmup.php';

/** @var Licence $L */
/** @var Plugin $P */
$response          = new stdClass();
$response->status  = 'success';
$response->data    = [];
$response->message = '';

if ( ! $L->canRun()) {
  $response->status  = 'error';
  $response->message = 'licence failure';
} else {
  $params = array_merge($_GET, $_POST);
  if ('application/json' === (getallheaders()['Content-Type'] ?? '')) {
    $params = array_merge($params, json_decode(file_get_contents('php://input'), true) ?? []);
  }

  $cinPlugin = $P->getJTlPlugin();

  switch ($_SERVER['REQUEST_METHOD'].'_'.($params['action'] ?? 'missingaction')) {
    case 'POST_checkSelection':

      if (($response->data['available'] = EventConfigService::eventAvailableCount($params['configID'])) > 0) {
        $_SESSION['cin_events_5']->configID = $params['configID'];

        $response->data['config'] = EventConfigService::getConfig($params['configID']);
        $response->data['matrix'] = new stdClass();

        $mD = EventMatrixService::getEventMatrixData($params['configID']);

        if ( ! $mD || (isset($mD[0]) && null === $mD[0]['identifier'])) {
          $response->data['matrix'] = null;
        } else {
          $response->data['matrix']->identifier = $mD[0]['identifier'];
          $response->data['matrix']->filename   = $mD[0]['filename'];
          $response->data['matrix']->svg        = file_get_contents(\dirname(__DIR__, 3).'/mediafiles/cin_events_5/matrix/'.$mD[0]['filename']);
          $response->data['matrix']->sold       = array_filter($mD, static function ($v) {
            return (bool) $v['sold'];
          });
          $response->data['matrix']->reserved = array_filter($mD, static function ($v) {
            return ! (bool) $v['sold'];
          });
          $response->data['matrix']->self = array_filter($mD, static function ($v) {
            return $v['user'] === $_SESSION['cin_events_5']->user;
          });

          $response->data['matrix']->vars = EventMatrixService::getMatrixVar($params['configID']);
        }
      } else {
        $_SESSION['cin_events_5']->configID = null;
      }

      break;

    case 'POST_loadDays':

      $response->data = EventConfigService::getEventDays($params['eventID'], false);

      break;

    case 'POST_loadEvent':

      date_default_timezone_set('Europe/Berlin');

      $response->data = array_values(array_filter(
        EventService::loadEvents(),
        static function ($v, $k) use ($params) {
          return $v['kArtikel'] === $params['eventID'];
        },
        \ARRAY_FILTER_USE_BOTH
      ));

      $additional = EventService::getEventData($params['eventID'], false);
      array_walk($additional, static function (&$array): void {
        foreach ($array as &$set) {
          if (978303600 === strtotime('2001-01-01, '.$set['begin'])) {
            $set['begin'] = null;
          }
        }
      });

      array_walk($response->data, static function (&$array) use ($additional): void {
        $array['dates'] = $additional;
        // $array['dates'] = [];
        // for($i = count($additional)-1; $i >= 0; $i--) {
        // if ($array['kArtikel'] == $additional[$i]['kVaterArtikel']) $array['dates'][] = array_splice($additional, $i, 1)[0];
        // }
      });

      break;

    case 'POST_seatReservation':
      // $position_id      = (string) $params['seat'] ?: 'PLACELESS_'.EventService::GUIDv4();
      $response->data   = EventMatrixService::getMatrixVar($params['configID']);
      $response->status = EventCartService::reserveSeat((int) $params['configID'], (string) $params['seat'], $_SESSION['cin_events_5']->user)
        ? 'success' : 'error';

      break;

    case 'DELETE_seatReservation':

      $response->data   = EventMatrixService::getMatrixVar($params['configID']);
      $response->status = EventCartService::deleteReservedSeat((int) $params['configID'], (string) $params['seat'], $_SESSION['cin_events_5']->user)
      ? 'success' : 'error';

      break;

    case 'GET_template':

      $response->data = EventService::getTemplates($params['type']);

      break;
  }
}

header('Content-Type: application/json');
exit(json_encode($response, 512));
