<?php

declare(strict_types=1);

namespace Cin\Events5\classes;

use stdClass;

/**
 * Response object with member from description.
 *
 * @param string   $status  Manage status like success and error. On merging of Response objects, 'error' is always forced.
 * @param string   $message on merging of ServiceResponse objects, different messages will be add as new lines
 * @param stdClass $data    Data object for free handling. On merging with non empty data objects, the first object become the member slice as array with all objects.
 */
final class ServiceResponse
{
  public string $status;

  public string $message;

  public stdClass $data;

  public function __construct(string $status = 'success', ?string $message = null, ?stdClass $data = null)
  {
    $this->status  = $status;
    $this->message = $message ?? '';
    $this->data    = $data ?? new stdClass();
  }

  /**
   * Merge two ServiceResponse objects.
   * - Has one object status 'error', this status will be forced.
   * - Different messages will be added as new lines.
   * - Merging non empty data objects, the first object become the member slice as array with all objects.
   *
   * @param ServiceResponse $serviceResponse the new Response object that is to merge with this
   */
  public function merge(self $serviceResponse): void
  {
    if ('error' === $serviceResponse->status) {
      $this->status = 'error';
    }

    if (\mb_strlen($serviceResponse->message) > 0 && false === mb_strpos($this->message, $serviceResponse->message)) {
      $this->message .= (\mb_strlen($this->message) ? "\n" : '').$serviceResponse->message;
    }

    if (\count((array) $serviceResponse->data) > 0) {
      if (0 === \count((array) $this->data)) {
        $this->data = $serviceResponse->data;
      } elseif (isset($this->data->slice)) {
        $this->data->slice[] = $serviceResponse->data;
      } else {
        $this->data->slice[] = $this->data;
        $this->data->slice[] = $serviceResponse->data;
      }
    }
  }

  public function status(bool $status): self
  {
    $this->status = $status ? 'success' : 'error';

    return $this;
  }
}
