<?php

declare(strict_types=1);

namespace Cin\Events5\service;

use Cin\Events5\Plugin;
use JTL\Catalog\Product\Artikel;
use JTL\Shop;
use JTL\Smarty\JTLSmarty;
use PDO;
use Smarty;
use stdClass;

final class EventService
{
  public const EVENT_NO_IMAGE = '/gfx/keinBild.gif';

  public const EVENT_USER_IMAGE_PATH = '/mediafiles/cin_events_5/';

  private static $plugin;

  public static function deleteEventImage($kVaterArtikel): void
  {
    $db = Shop::Container()->getDB();

    $stmt = "UPDATE cin_events_5 SET image = '' WHERE kVaterArtikel = :kVaterArtikel";

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':kVaterArtikel' => $kVaterArtikel]);
  }

  /**
   * Ensures that the event is already created.
   */
  public static function ensureEvent(int $kVaterArtikel): void
  {
    $event = self::getEvent($kVaterArtikel);

    if (empty((array) $event)) {
      $stmt = 'INSERT INTO cin_events_5 (kVaterArtikel) VALUES (:kVaterArtikel)';

      $prep = Shop::Container()->getDB()->getPDO()->prepare($stmt);
      $prep->execute([':kVaterArtikel' => $kVaterArtikel]);
    }
  }

  public static function getEventData($kVaterArtikel, bool $forceAllDays = true): array
  {
    $db = Shop::Container()->getDB();

    $stmt = "SELECT #DISTINCT
      c.`date` 'date',
      e.kVaterArtikel kVaterArtikel,
      e.image eventImage,
      c.`begin` 'begin',
      c.`end` 'end',
      c.id config,
      c.contingent,
      SUM(CASE WHEN r.sold = 0 THEN r.count ELSE 0 END) reserved,
      SUM(CASE WHEN r.sold = 1 THEN r.count ELSE 0 END) sold,
      c.matrix_id
      FROM cin_events_5 e
      LEFT JOIN cin_events_5_config c ON c.events_id = e.kVaterArtikel
      LEFT JOIN cin_events_5_reserved r ON r.config_id = c.id
      WHERE e.kVaterArtikel = :kVaterArtikel
      AND c.id IS NOT NULL
      AND c.`date` >= :date
      GROUP BY c.id
      ORDER BY c.`date`, c.`begin`";

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([
      ':kVaterArtikel' => $kVaterArtikel,
      ':date'          => $forceAllDays ? '2001-01-01' : date('Y-m-d')
    ]);

    return $prep->fetchAll(PDO::FETCH_ASSOC | PDO::FETCH_GROUP);
  }

  public static function getTemplates(string $type, $smarty = null): object
  {
    $tpls = new stdClass();
    $P    = new Plugin();
    $jP   = $P->getJTlPlugin();

    switch ($type) {
      case 'admin':

        $adminPath = $jP->getPaths()->getAdminPath().'template/';

        $tpls->date    = file_get_contents($adminPath.'event_dates.tpl');
        $tpls->time    = file_get_contents($adminPath.'event_times.tpl');
        $tpls->mtxOpt  = file_get_contents($adminPath.'matrix/option.tpl');
        $tpls->linker  = $smarty->fetch($adminPath.'link_vars.tpl');
        $tpls->mtxItem = self::prepareTemplate(file_get_contents($adminPath.'matrix/item.tpl'), $P);
        $tpls->notify  = self::prepareTemplate(file_get_contents($jP->getPaths()->getBasePath().'/common/template/notify.tpl'), $P);

        break;

      case 'adminCalendar':

        $adminPath = $jP->getPaths()->getAdminPath().'template/adminCalendar/';

        $tpls->prevHtml = file_get_contents($adminPath.'prevHtml.tpl');
        $tpls->nextHtml = file_get_contents($adminPath.'nextHtml.tpl');
        $tpls->calendar = file_get_contents($adminPath.'calendar.tpl');

        break;

      case 'product':

        $frontendPath = $jP->getPaths()->getFrontendPath().'template/';

        $tpls->group = file_get_contents($frontendPath.'product-event-slider-item.tpl');
        $tpls->item  = file_get_contents($frontendPath.'product-event-slider-item-inner.tpl');

        break;
    }

    return $tpls;
  }

  public static function getVariation($eventID)
  {
    $artikelOptions                    = new stdClass();
    $artikelOptions->nArtikelAttribute = 0;
    $artikelOptions->nAttribute        = 0;
    $artikelOptions->nDownload         = 0;
    $artikelOptions->nKonfig           = 0;
    $artikelOptions->nMerkmale         = 0;
    $artikelOptions->nVariationen      = 1;

    $artikel = (new Artikel())->fuelleArtikel((int) $eventID, $artikelOptions);

    $rslt = [];

    if (1 === \count($artikel->VariationenOhneFreifeld)) {
      foreach ($artikel->VariationenOhneFreifeld as $variation) {
        foreach ($variation->Werte as $value) {
          $rslt[$value->cName] = '_'.$value->kEigenschaft.':'.$value->kEigenschaftWert;
        }
      }
    } else {
      $tmp = [];
      $i   = 0;
      foreach ($artikel->VariationenOhneFreifeld as $variation) {
        foreach ($variation->Werte as $value) {
          $tmp[$i][$value->cName] = $value->kEigenschaft.':'.$value->kEigenschaftWert;
        }
        ++$i;
      }

      if (\count($tmp) > 1) {
        foreach ($tmp[0] as $name => $value) {
          foreach ($tmp[1] as $name2 => $value2) {
            $rslt[$name.'-'.$name2] = $value.'_'.$value2;
            // $rslt[$name2.'-'.$name] = $value2.'_'.$value;
          }
        }
      }
    }

    return $rslt;
  }

  public static function generateRandomString($length = 10)
  {
    $characters       = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $charactersLength = mb_strlen($characters);
    $randomString     = '';
    for ($i = 0; $i < $length; ++$i) {
      $randomString .= $characters[mt_rand(0, $charactersLength - 1)];
    }

    return $randomString;
  }

  /**
   * Erstellt eine Global eindeutige ID.
   */
  public static function GUIDv4(bool $trim = true): string
  {
    // Windows
    if (true === \function_exists('com_create_guid')) {
      if (true === $trim) {
        return trim(com_create_guid(), '{}');
      }

      return com_create_guid();
    }

    // OSX/Linux
    if (true === \function_exists('openssl_random_pseudo_bytes')) {
      $data    = openssl_random_pseudo_bytes(16);
      $data[6] = \chr(\ord($data[6]) & 0x0f | 0x40);    // set version to 0100
      $data[8] = \chr(\ord($data[8]) & 0x3f | 0x80);    // set bits 6-7 to 10

      return vsprintf('%s%s-%s-%s-%s-%s%s%s', mb_str_split(bin2hex($data), 4));
    }

    // Fallback (PHP 4.2+)
    mt_srand((int) microtime(true) * 10000);
    $charid = mb_strtolower(md5(uniqid((string) mt_rand(), true)));
    $hyphen = \chr(45);                  // "-"
    $lbrace = $trim ? '' : \chr(123);    // "{"
    $rbrace = $trim ? '' : \chr(125);    // "}"
    $guidv4 = $lbrace
    .mb_substr($charid, 0, 8).$hyphen
    .mb_substr($charid, 8, 4).$hyphen
    .mb_substr($charid, 12, 4).$hyphen
    .mb_substr($charid, 16, 4).$hyphen
    .mb_substr($charid, 20, 12)
    .$rbrace;

    return $guidv4;
  }

  public static function handleUserId(): string
  {
    if ( ! isset($_SESSION['cin_events_5'])) {
      $_SESSION['cin_events_5'] = new stdClass();
    }

    $sess       = $_SESSION['cin_events_5'];
    $sess->user = $sess->user ?? self::GUIDv4();
    $kdnr       = isset($_SESSION['Kunde'], $_SESSION['Kunde']->cKundenNr) && null !== $_SESSION['Kunde']->cKundenNr ? $_SESSION['Kunde']->cKundenNr : false;

    if ($kdnr && $kdnr !== $sess->user) {
      $pdo  = Shop::Container()->getDB()->getPDO();
      $prep = $pdo->prepare('UPDATE cin_events_5_reserved SET identifier = :new WHERE identifier = :old');
      $prep->execute([
        ':new' => $kdnr,
        ':old' => $sess->user
      ]);

      $sess->user = $kdnr;
    }

    return $sess->user;
  }

  public static function loadEvents($pdo_fetch_style = PDO::FETCH_ASSOC)
  {
    $db = Shop::Container()->getDB();

    $stmt = "SELECT
      taa.kArtikel kArtikel,
      ta.cName Artikel,
      ta.cSeo Link,
      ta.cKurzBeschreibung description,
      tk.cName Kategorie,
      tk.kKategorie,
      tk.kOberKategorie,
      ce.image 'image'
      FROM tartikelattribut taa
      LEFT JOIN tartikel ta ON ta.kArtikel = taa.kArtikel
      LEFT JOIN tkategorieartikel tka ON tka.kArtikel = taa.kArtikel
      LEFT JOIN tkategorie tk ON tk.kKategorie = tka.kKategorie
      LEFT JOIN cin_events_5 ce ON ce.kVaterArtikel = taa.kArtikel
      WHERE taa.cName LIKE 'cin_event'";

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute();

    return $prep->fetchAll($pdo_fetch_style);
  }

  public static function loadEventStructure($pdo_fetch_style = PDO::FETCH_ASSOC)
  {
    $db = Shop::Container()->getDB();

    $stmt = "SELECT
      e.kVaterArtikel kVaterArtikel,
      #e.id evt_ID,
      e.image eventImage,
      #d.id days_ID,
      #d.kArtikel kVarArtikel,
      c.`date` 'date',
      #t.id times_ID,
      c.`begin` 'begin',
      c.`end` 'end'
      FROM cin_events_5 e
      LEFT JOIN cin_events_5_config c ON c.events_id = e.kVaterArtikel
      ORDER BY c.`date`, c.`begin`";

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute();

    return $prep->fetchAll($pdo_fetch_style);
  }

  public static function loadExtendedEvents($pdo_fetch_style = PDO::FETCH_ASSOC)
  {
    $db = Shop::Container()->getDB();

    $stmt = "select
      father.kArtikel 'father',
      child.kArtikel 'child'
      from tartikel father
      left join tartikelattribut taa on taa.kArtikel = father.kArtikel
      left join tartikel child on child.kVaterArtikel = father.kArtikel
      WHERE taa.cName LIKE 'cin_event'";

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute();

    return $prep->fetchAll($pdo_fetch_style);
  }

  public static function storeEventImage(int $kVaterArtikel, string $identifier): void
  {
    self::ensureEvent($kVaterArtikel);

    $db = Shop::Container()->getDB();

    $stmt = 'INSERT INTO cin_events_5 (kVaterArtikel, image)
      VALUES(:kVaterArtikel, :identifier)
      ON DUPLICATE KEY UPDATE image=:identifier;';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':kVaterArtikel' => $kVaterArtikel, ':identifier' => $identifier]);
  }

  public static function Plugin(): Plugin
  {
    return self::$plugin ?? (self::$plugin = new Plugin());
  }

  private static function getEvent($kVaterArtikel): stdClass
  {
    $stmt = 'SELECT * FROM cin_events_5 WHERE kVaterArtikel = :kVaterArtikel';

    $prep = Shop::Container()->getDB()->getPDO()->prepare($stmt);
    $prep->execute([':kVaterArtikel' => $kVaterArtikel]);

    return $prep->fetch(PDO::FETCH_OBJ) ?: new stdClass();
  }

  /**
   * prepare smarty templates for ajax requests. Replace params and language params will be set. Includes like {{include ...}} are possible.
   */
  private static function prepareTemplate(string $snippet, Plugin $P): string
  {
    // fetch includes
    $snippet = preg_replace_callback("/{include\\s*'(.+)'}/", static function ($replace) use (&$count, $P) {
      return \is_array($replace) && \count($replace) >= 2 ? file_get_contents($P->getJTlPlugin()->getPaths()->getFrontendPath().'/template/'.$replace[1]) : '';
    }, $snippet);

    // line break fix for patterns
    $snippet = preg_replace('/(\r\n|\r|\n)/', '', $snippet);
    $snippet = preg_replace('/{\*\/repl\*}/', '{*/repl*}'."\n", $snippet);

    $count   = 0;
    $snippet = preg_replace('/{[^?_*][^{]*(?<!translate)}/', '<!-- replace -->', preg_replace('/{\*repl\*}.*{\*\/repl\*}/', '<!-- replace -->', $snippet));

    $smarty = new JTLSmarty();
    $smarty->registerPlugin(
      Smarty::PLUGIN_MODIFIER,
      'translate',
      [$P, 'translate']
    );
    $snippet = $smarty->fetch('string:'.$snippet);

    return preg_replace_callback('/<!-- replace -->/', static function () use (&$count) {
      return '{'.$count++.'}';
    }, $snippet);
  }
}
