<?php

declare(strict_types=1);

namespace ComInsNetz\FileDropper;

use PDO;

final class FileDropper
{
  private PDO $PDO;

  public function __construct(PDO $PDO)
  {
    $this->PDO = $PDO;
  }

  public function storeAsPNG(string $filePath, ?string $name = null): bool
  {
    $name       = $name ?? pathinfo($_FILES['uploadFile']['name'], \PATHINFO_FILENAME);
    $fileData   = imagecreatefromstring(file_get_contents($_FILES['uploadFile']['tmp_name']));
    $dimensions = self::getSmallImageDimension($this->PDO);
    $dimMax     = max((int) $dimensions->w, (int) $dimensions->h);
    $gdImage    = self::scaleMax($fileData, (int) $dimMax);
    $gdImage    = self::paddToSquare($gdImage);

    return imagepng($gdImage, rtrim($filePath, '\\/').\DIRECTORY_SEPARATOR.$name.'.png');
  }

  public static function getImportString(): string
  {
    $root  = \dirname(__FILE__, 5);
    $short = \dirname(mb_substr(__FILE__, mb_strlen($root)));

    return str_replace('\\', '/', '<script type="text/javascript" src="/plugins/%1$s'.$short.'/FileDropper.js"></script>
    <link rel="stylesheet" type="text/css" href="/plugins/%1$s'.$short.'/FileDropper.css">');
  }

  public static function getSmallImageDimension(PDO $PDO): object
  {
    $stmt = $PDO->prepare("SELECT
      cWert AS w,
      (
        SELECT cWert
        FROM teinstellungen
        WHERE cName = 'bilder_artikel_klein_hoehe'
      ) as h
      FROM teinstellungen
      WHERE cName = 'bilder_artikel_klein_breite'");
    $stmt->execute();

    return $stmt->fetch(PDO::FETCH_OBJ);
  }

  private static function dims($img)
  {
    $width  = imagesx($img);
    $height = imagesy($img);

    return [
      $width,
      $height
    ];
  }

  private static function paddToSquare($imgSrc)
  {
    $srcDims = self::dims($imgSrc);
    $size    = max($srcDims);
    $imgTrg  = imagecreatetruecolor($size, $size);
    $alpha   = imagecolorallocatealpha($imgTrg, 0, 0, 0, 127);
    imagefill($imgTrg, 0, 0, $alpha);
    imagealphablending($imgTrg, false);
    imagesavealpha($imgTrg, true);
    $offsetX = ($srcDims[1] - $srcDims[0]) / 2;
    $offsetY = 0;
    if ($srcDims[0] > $srcDims[1]) {
      $offsetX = 0;
      $offsetY = ($srcDims[0] - $srcDims[1]) / 2;
    }
    imagecopy($imgTrg, $imgSrc, $offsetX, $offsetY, 0, 0, $srcDims[0], $srcDims[1]);

    return $imgTrg;
  }

  private static function scaleMax($imgSrc, $dimTrg)
  {
    imagealphablending($imgSrc, true);
    imagesavealpha($imgSrc, true);

    $dimsImg = self::dims($imgSrc);
    $ratImg  = $dimsImg[0] / $dimsImg[1];
    $dimsTrg = [];

    if ($ratImg <= 1) {
      $dimsTrg[1] = $dimTrg;
      $dimsTrg[0] = $ratImg * $dimTrg;
    }

    if ($ratImg > 1) {
      $dimsTrg[0] = $dimTrg;
      $dimsTrg[1] = $dimTrg / $ratImg;
    }

    $imgTrg = imagecreatetruecolor($dimsTrg[0], $dimsTrg[1]);
    $color  = imagecolorallocatealpha($imgTrg, 0, 0, 0, 127);
    imagefill($imgTrg, 0, 0, $color);
    imagealphablending($imgTrg, false);
    imagesavealpha($imgTrg, true);
    imagecopyresampled($imgTrg, $imgSrc, 0, 0, 0, 0, $dimsTrg[0], $dimsTrg[1], $dimsImg[0], $dimsImg[1]);
    imagesavealpha($imgTrg, true);

    return $imgTrg;
  }
}
